@@ -30,6 +30,7 @@ SYSTEM_TASKS += ['struct_info']
 
 USER_TASKS = [
     'binaryen',
+    'boost_headers',
     'bullet',
     'bzip2',
     'cocos2d',
@@ -45,6 +46,7 @@ USER_TASKS = [
     'sdl2-gfx',
     'sdl2-image',
     'sdl2-image-png',
+    'sdl2-image-jpg',
     'sdl2-mixer',
     'sdl2-net',
     'sdl2-ttf',
@@ -54,6 +56,7 @@ USER_TASKS = [
 
 temp_files = shared.configuration.get_temp_files()
 logger = logging.getLogger('embuilder')
+force = False
 
 
 def print_help():
@@ -81,8 +84,9 @@ Issuing 'embuilder.py build ALL' causes each task to be built.
 
 Flags:
 
-  --lto  Build bitcode files, for LTO with the LLVM wasm backend
-  --pic  Build as position independent code (used by MAIN_MODULE/SIDE_MODULE)
+  --force  Force rebuild of target (by removing it first)
+  --lto    Build bitcode files, for LTO with the LLVM wasm backend
+  --pic    Build as position independent code (used by MAIN_MODULE/SIDE_MODULE)
 
 It is also possible to build native_optimizer manually by using CMake. To
 do that, run
@@ -106,6 +110,10 @@ def build(src, result_libs, args=[]):
   cpp = os.path.join(temp_dir, 'src.cpp')
   open(cpp, 'w').write(src)
   temp_js = os.path.join(temp_dir, 'out.js')
+  if force:
+    for lib in result_libs:
+      shared.Cache.erase_file(lib)
+
   try:
     shared.Building.emcc(cpp, args, output_filename=temp_js)
   except subprocess.CalledProcessError as e:
@@ -121,6 +129,8 @@ def build_port(port_name, lib_name, params):
 
 
 def main():
+  global force
+
   if len(sys.argv) < 2 or sys.argv[1] in ['-v', '-help', '--help', '-?', '?']:
     print_help()
     return 0
@@ -148,6 +158,8 @@ def main():
         shared.Settings.WASM_OBJECT_FILES = 0
       elif arg == 'pic':
         shared.Settings.RELOCATABLE = 1
+      elif arg == 'force':
+        force = True
       # Reconfigure the cache dir to reflect the change
       shared.reconfigure_cache()
 
@@ -190,6 +202,8 @@ def main():
     logger.info('building and verifying ' + what)
     if what in SYSTEM_LIBRARIES:
       library = SYSTEM_LIBRARIES[what]
+      if force:
+        library.erase()
       library.get_path()
     elif what == 'struct_info':
       build(C_BARE, ['generated_struct_info.json'])
@@ -221,6 +235,8 @@ def main():
       build_port('sdl2-image', libname('libSDL2_image'), ['-s', 'USE_SDL=2', '-s', 'USE_SDL_IMAGE=2'])
     elif what == 'sdl2-image-png':
       build_port('sdl2-image', libname('libSDL2_image'), ['-s', 'USE_SDL=2', '-s', 'USE_SDL_IMAGE=2', '-s', 'SDL2_IMAGE_FORMATS=["png"]'])
+    elif what == 'sdl2-image-jpg':
+      build_port('sdl2-image', libname('libSDL2_image'), ['-s', 'USE_SDL=2', '-s', 'USE_SDL_IMAGE=2', '-s', 'SDL2_IMAGE_FORMATS=["jpg"]'])
     elif what == 'sdl2-net':
       build_port('sdl2-net', libname('libSDL2_net'), ['-s', 'USE_SDL=2', '-s', 'USE_SDL_NET=2'])
     elif what == 'sdl2-mixer':
@@ -237,6 +253,8 @@ def main():
       build_port('cocos2d', libname('libcocos2d'), ['-s', 'USE_COCOS2D=3', '-s', 'USE_ZLIB=1', '-s', 'USE_LIBPNG=1', '-s', 'ERROR_ON_UNDEFINED_SYMBOLS=0'])
     elif what == 'regal':
       build_port('regal', libname('libregal'), ['-s', 'USE_REGAL=1'])
+    elif what == 'boost_headers':
+      build_port('boost_headers', libname('libboost_headers'), ['-s', 'USE_BOOST_HEADERS=1'])
     else:
       logger.error('unfamiliar build target: ' + what)
       return 1
