@@ -4,8 +4,7 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-'''
-This is a helper script. It runs make for you, setting
+"""This is a helper script. It runs make for you, setting
 the environment variables to use emcc and so forth. Usage:
 
   emmake make [FLAGS]
@@ -20,18 +19,19 @@ The difference between this and emconfigure is that
 emconfigure runs compilation into native code, so
 that configure tests pass. emmake uses Emscripten to
 generate JavaScript.
-'''
+"""
 
 from __future__ import print_function
-import os, sys
+import sys
 from tools import shared
 from subprocess import CalledProcessError
 
+
 #
 # Main run() function
 #
 def run():
-  if len(sys.argv) < 2 or sys.argv[1] != 'make':
+  if len(sys.argv) < 2 or sys.argv[1] in ('--version', '--help'):
     print('''
   emmake is a helper for make, setting various environment
   variables so that emcc etc. are used. Typical usage:
@@ -41,11 +41,15 @@ def run():
   (but you can run any command instead of make)
 
   ''', file=sys.stderr)
+    return 1
 
   try:
     shared.Building.make(sys.argv[1:])
   except CalledProcessError as e:
-    sys.exit(e.returncode)
+    return e.returncode
+
+  return 0
+
 
 if __name__ == '__main__':
-  run()
+  sys.exit(run())
