@@ -4,7 +4,7 @@
 // found in the LICENSE file.
 
 //
-// Various compiling-to-JS parameters. These are simply variables present when the
+// Various compiler settings. These are simply variables present when the
 // JS compiler runs. To set them, do something like:
 //
 //   emcc -s OPTION1=VALUE1 -s OPTION2=VALUE2 [..other stuff..]
@@ -23,6 +23,29 @@
 // should not be a need to have them when just compiling source to
 // bitcode. However, there will also be no harm either, so it is ok to.
 //
+// Settings in this file can be directly set from the command line.  Internal
+// settings that are not part of the user ABI live in the settings_internal.js.
+//
+// In general it is best to pass the same arguments at both compile and link
+// time, as whether wasm object files are used or not affects when codegen
+// happens (without wasm object files, or when using fastcomp, codegen is all
+// during link; otherwise, it is during compile). Flags affecting codegen must
+// be passed when codegen happens, so to let a build easily switch when codegen
+// happens (LTO vs normal), pass the flags at both times. The flags are also
+// annotated in this file:
+//
+// [link] - Should be passed at link time. This is the case for all JS flags,
+//          as we emit JS at link (and that is most of the flags here, and
+//          hence the default).
+// [compile+link] - A flag that has an effect at both compile and link time,
+//                  basically any time emcc is invoked. The same flag should be
+//                  passed at both times in most cases.
+//
+// If not otherwise specified, a flag is [link]. Note that no flag is only
+// relevant during compile time, as during link we may do codegen for system
+// libraries and other support code, so all flags are either link or
+// compile+link.
+//
 
 // Tuning
 
@@ -213,17 +236,11 @@ var DECLARE_ASM_MODULE_EXPORTS = 1;
 // Ignore closure warnings and errors (like on duplicate definitions)
 var IGNORE_CLOSURE_COMPILER_ERRORS = 0;
 
-// When enabled, does not push/pop the stack at all in functions that have no
-// basic stack usage. But, they may allocate stack later, and in a loop, this
-// can be very bad. In particular, when debugging, printf()ing a lot can exhaust
-// the stack very fast, with this option.  In particular, be careful with the
-// autodebugger! (We do turn this off automatically in that case, though.)
-var SKIP_STACK_IN_SMALL = 1;
-
 // A limit on inlining. If 0, we will inline normally in LLVM and closure. If
 // greater than 0, we will *not* inline in LLVM, and we will prevent inlining of
 // functions of this size or larger in closure. 50 is a reasonable setting if
 // you do not want inlining
+// [compile+link]
 var INLINING_LIMIT = 0;
 
 // Run aggressiveVariableElimination in js-optimizer.js
@@ -242,6 +259,10 @@ var SAFE_HEAP = 0;
 // Log out all SAFE_HEAP operations
 var SAFE_HEAP_LOG = 0;
 
+// Check each stack pointer decrement on WASM backend to ensure that the stack
+// does not overflow.
+var SAFE_STACK = 0;
+
 // In asm.js mode, we cannot simply add function pointers to function tables, so
 // we reserve some slots for them. An alternative to this is to use
 // EMULATED_FUNCTION_POINTERS, in which case we don't need to reserve.
@@ -300,7 +321,10 @@ var DEMANGLE_SUPPORT = 0;
 //   emscripten_run_script("Runtime.debug = ...;");
 var LIBRARY_DEBUG = 0;
 
-// Print out all syscalls
+// Print out all musl syscalls, including translating their numeric index
+// to the string name, which can be convenient for debugging. (Other system
+// calls are not numbered and already have clear names; use LIBRARY_DEBUG
+// to get logging for all of them.)
 var SYSCALL_DEBUG = 0;
 
 // Log out socket/network data transfer.
@@ -328,6 +352,7 @@ var WEBSOCKET_URL = 'ws://';
 
 // A string containing a comma separated list of WebSocket subprotocols
 // as would be present in the Sec-WebSocket-Protocol header.
+// You can set 'null', if you don't want to specify it.
 var WEBSOCKET_SUBPROTOCOL = 'binary';
 
 // Print out debugging information from our OpenAL implementation.
@@ -453,6 +478,12 @@ var STB_IMAGE = 0;
 // If you do not care about old iOS 9 support, keep this disabled.
 var WORKAROUND_IOS_9_RIGHT_SHIFT_BUG = 0;
 
+// From Safari 8 (where WebGL was introduced to Safari) onwards, OES_texture_half_float and OES_texture_half_float_linear extensions
+// are broken and do not function correctly, when used as source textures.
+// See https://bugs.webkit.org/show_bug.cgi?id=183321, https://bugs.webkit.org/show_bug.cgi?id=169999,
+// https://stackoverflow.com/questions/54248633/cannot-create-half-float-oes-texture-from-uint16array-on-ipad
+var GL_DISABLE_HALF_FLOAT_EXTENSION_IF_BROKEN = 0;
+
 // If set, enables polyfilling for Math.clz32, Math.trunc, Math.imul, Math.fround.
 var POLYFILL_OLD_MATH_FUNCTIONS = 0;
 
@@ -508,10 +539,13 @@ var LZ4 = 0;
 //     -fno-exceptions to really get rid of all exceptions code overhead,
 //     as it may contain thrown exceptions that are never caught (e.g.
 //     just using std::vector can have that). -fno-rtti may help as well.
+//
+// [compile+link] - affects user code at compile and system libraries at link
 var DISABLE_EXCEPTION_CATCHING = 1;
 
 // Enables catching exception in the listed functions only, if
 // DISABLE_EXCEPTION_CATCHING = 2 is set
+// [compile+link] - affects user code at compile and system libraries at link
 var EXCEPTION_CATCHING_WHITELIST = [];
 
 // By default we handle exit() in node, by catching the Exit exception. However,
@@ -519,60 +553,83 @@ var EXCEPTION_CATCHING_WHITELIST = [];
 // longer do that, and exceptions work normally, which can be useful for libraries
 // or programs that don't need exit() to work.
 
-// For more explanations of this option, please visit
-// https://github.com/emscripten-core/emscripten/wiki/Asyncify
+// Emscripten uses an ExitStatus exception to halt when exit() is called.
+// With this option, we prevent that from showing up as an unhandled
+// exception.
 var NODEJS_CATCH_EXIT = 1;
 
+// Catch unhandled rejections in node. Without this, node may print the error,
+// and that this behavior will change in future node, wait a few seconds, and
+// then exit with 0 (which hides the error if you don't read the log). With
+// this, we catch any unhandled rejection and throw an actual error, which will
+// make the process exit immediately with a non-0 return code.
+var NODEJS_CATCH_REJECTION = 1;
+
 // Whether to enable asyncify transformation
 // This allows to inject some async functions to the C code that appear to be sync
 // e.g. emscripten_sleep
+// On fastcomp this uses the Asyncify IR transform.
+// On upstream this uses the Asyncify pass in Binaryen. TODO: whitelist, coroutines
 var ASYNCIFY = 0;
 
-// Functions that call any function in the list, directly or indirectly
-var ASYNCIFY_FUNCTIONS = ['emscripten_sleep',
-                          'emscripten_wget',  // will be transformed
-                          'emscripten_yield'];
-// Functions in this list are never considered async, even if they appear in ASYNCIFY_FUNCTIONS
-var ASYNCIFY_WHITELIST = ['qsort',
-                          'trinkle', // In the asyncify transformation, any function that calls a function pointer is considered async
-                          '__toread', // This whitelist is useful when a function is known to be sync
-                          '__uflow',  // currently this link contains some functions in libc
-                          '__fwritex',
-                          'MUSL_vfprintf'];
-
-// Runs the binaryen "bysyncify" pass to transform sync code into async.
-// This is similar to asyncify and EMTERPRETIFY_ASYNC but works with the
-// wasm backend.
-//
-// Done:
-//  * Sleep support.
-//  * Emscripten APIs (emscripten_wget* and other sync APIs).
-//  * Synchronous fsync syscall.
-//  * Browser integration.
-//
-// Not done:
-//  * No whitelist/blacklist support (hopefully with the simpler model
-//    and lower overhead they may not be needed?)
-//  * No coroutine support.
-var BYSYNCIFY = 0;
-
 // The imports which can do a sync operation. If you add more you will need to
 // add them to here.
-var BYSYNCIFY_IMPORTS = ['emscripten_sleep', 'emscripten_wget', 'emscripten_wget_data', 'emscripten_idb_load', 'emscripten_idb_store', 'emscripten_idb_delete', 'emscripten_idb_exists', 'emscripten_idb_load_blob', 'emscripten_idb_store_blob', 'SDL_Delay', '__syscall118'];
+// If the names here do not include a "module." prefix, the module is assumed
+// to be "env".
+var ASYNCIFY_IMPORTS = [
+  'emscripten_sleep', 'emscripten_wget', 'emscripten_wget_data', 'emscripten_idb_load',
+  'emscripten_idb_store', 'emscripten_idb_delete', 'emscripten_idb_exists',
+  'emscripten_idb_load_blob', 'emscripten_idb_store_blob', 'SDL_Delay',
+  'emscripten_scan_registers', 'emscripten_lazy_load_code',
+  'wasi_unstable.fd_sync', '__wasi_fd_sync',
+];
 
-// Whether indirect calls can be on the stack during an unwind/rewind. If you know
-// they cannot, then setting this can be extremely helpful, as otherwise bysyncify
+// Whether indirect calls can be on the stack during an unwind/rewind.
+// If you know they cannot, then setting this can be extremely helpful, as otherwise asyncify
 // must assume an indirect call can reach almost everywhere.
-var BYSYNCIFY_IGNORE_INDIRECT = 0;
+var ASYNCIFY_IGNORE_INDIRECT = 0;
 
-// The size of the Bysyncify stack - the region used to store unwind/rewind info.
-// This must be large enough to store the call stack and locals. If it is too
+// The size of the asyncify stack - the region used to store unwind/rewind
+// info. This must be large enough to store the call stack and locals. If it is too
 // small, you will see a wasm trap due to executing an "unreachable" instruction.
 // In that case, you should increase this size.
-var BYSYNCIFY_STACK_SIZE = 4096;
+var ASYNCIFY_STACK_SIZE = 4096;
 
-// Runtime debug logging from bysyncify internals.
-var BYSYNCIFY_DEBUG = 0;
+// If the Asyncify blacklist is provided, then the functions in it will not
+// be instrumented even if it looks like they need to. This can be useful
+// if you know things the whole-program analysis doesn't, like if you
+// know certain indirect calls are safe and won't unwind. But if you
+// get the list wrong things will break (and in a production build user
+// input might reach code paths you missed during testing, so it's hard
+// to know you got this right), so this is not recommended unless you
+// really know what are doing, and need to optimize every bit of speed
+// and size.
+// The names in this list are names from the WebAssembly Names section. The
+// wasm backend will emit those names in *human-readable* form instead of
+// typical C++ mangling. For example, you should write Struct::func()
+// instead of _ZN6Struct4FuncEv. C is also different from C++, as C
+// names don't end with parameters; as a result foo(int) in C++ would appear
+// as just foo in C (C++ has parameters because it needs to differentiate
+// overloaded functions). You will see warnings in the console if a name in the
+// list is missing (these are not errors because inlining etc. may cause
+// changes which would mean a single list couldn't work for both -O0 and -O1
+// builds, etc.). You can inspect the wasm binary to look for the actual names,
+// either directly or using wasm-objdump or wasm-dis, etc.
+// Simple '*' wildcard matching is supported.
+var ASYNCIFY_BLACKLIST = [];
+
+// If the Asyncify whitelist is provided, then *only* the functions in the list
+// will be instrumented. Like the blacklist, getting this wrong will break
+// your application.
+// See notes on ASYNCIFY_BLACKLIST about the names.
+var ASYNCIFY_WHITELIST = [];
+
+// Allows lazy code loading: where emscripten_lazy_load_code() is written, we
+// will pause execution, load the rest of the code, and then resume.
+var ASYNCIFY_LAZY_LOAD_CODE = 0;
+
+// Runtime debug logging from asyncify internals.
+var ASYNCIFY_DEBUG = 0;
 
 // Runtime elements that are exported on Module by default. We used to export
 // quite a lot here, but have removed them all, so this option is redundant
@@ -592,6 +649,43 @@ var EXPORTED_RUNTIME_METHODS = [];
 // this list lets you add to the default list without modifying it.
 var EXTRA_EXPORTED_RUNTIME_METHODS = [];
 
+// A list of incoming values on the Module object in JS that we care about. If
+// a value is not in this list, then we don't emit code to check if you provide
+// it on the Module object. For example, if
+// you have this:
+//
+//  var Module = {
+//    print: function(x) { console.log('print: ' + x) },
+//    preRun: [function() { console.log('pre run') }]
+//  };
+//
+// Then MODULE_JS_API must contain 'print' and 'preRun'; if it does not then
+// we may not emit code to read and use that value. In other words, this
+// option lets you set, statically at compile time, the list of which Module
+// JS values you will be providing at runtime, so the compiler can better
+// optimize.
+//
+// Setting this list to [], or at least a short and concise set of names you
+// actually use, can be very useful for reducing code size. By default the
+// list contains all the possible APIs.
+//
+// FIXME: should this just be  0  if we want everything?
+var INCOMING_MODULE_JS_API = [
+  'ENVIRONMENT', 'GL_MAX_TEXTURE_IMAGE_UNITS', 'SDL_canPlayWithWebAudio',
+  'SDL_numSimultaneouslyQueuedBuffers', 'TOTAL_MEMORY', 'wasmMemory', 'arguments',
+  'buffer', 'canvas', 'doNotCaptureKeyboard', 'dynamicLibraries',
+  'elementPointerLock', 'extraStackTrace', 'forcedAspectRatio',
+  'instantiateWasm', 'keyboardListeningElementfreePreloadedMediaOnUse',
+  'locateFile', 'logReadFiles', 'mainScriptUrlOrBlob', 'mem',
+  'monitorRunDependencies', 'noExitRuntime', 'noInitialRun', 'onAbort',
+  'onCustomMessage', 'onExit', 'onFree', 'onFullScreen', 'onMalloc',
+  'onRealloc', 'onRuntimeInitialized', 'postMainLoop', 'postRun', 'preInit',
+  'preMainLoop', 'preRun',
+  'preinitializedWebGLContextmemoryInitializerRequest', 'preloadPlugins',
+  'print', 'printErr', 'quit', 'setStatus', 'statusMessage', 'stderr',
+  'stdin', 'stdout', 'thisProgram', 'wasm', 'wasmBinary', 'websocket'
+];
+
 // If set to nonzero, the provided virtual filesystem if treated
 // case-insensitive, like Windows and macOS do. If set to 0, the VFS is
 // case-sensitive, like on Linux.
@@ -611,9 +705,10 @@ var FILESYSTEM = 1;
 // some JS that does, you might need this.
 var FORCE_FILESYSTEM = 0;
 
-// This mode is intended for use with Node.js (and will throw if the build runs
-// in other engines).  The File System API will directly use Node.js API without
-// requiring `FS.mount()`.  The initial working directory will be same as
+// Enables support for the NODERAWFS filesystem backend. This is a special
+// backend as it replaces all normal filesystem access with direct Node.js
+// operations, without the need to do `FS.mount()`, and this backend only
+// works with Node.js. The initial working directory will be same as
 // process.cwd() instead of VFS root directory.  Because this mode directly uses
 // Node.js to access the real local filesystem on your OS, the code will not
 // necessarily be portable between OSes - it will be as portable as a Node.js
@@ -661,12 +756,12 @@ var RETAIN_COMPILER_SETTINGS = 0;
 // just functions. For example, you can include the Browser object by adding
 // "$Browser" to this list.
 var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [
-	'memcpy',
-	'memset',
-	'malloc',
-	'free',
-	'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
-	];
+  'memcpy',
+  'memset',
+  'malloc',
+  'free',
+  'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
+];
 
 // This list is also used to determine auto-exporting of library dependencies
 // (i.e., functions that might be dependencies of JS library functions, that if
@@ -747,12 +842,17 @@ var LINKABLE = 0;
 // Set the environment variable EMCC_STRICT=1 or pass -s STRICT=1 to test that a
 // codebase builds nicely in forward compatible manner.
 // Changes enabled by this:
-//   * ERROR_ON_MISSING_LIBRARIES is enabled
-//   * DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR is enabled
+//   * DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR is enabled.
 //   * The C define EMSCRIPTEN is not defined (__EMSCRIPTEN__ always is, and
 //     is the correct thing to use).
+//   * STRICT_JS is enabled.
+//   * AUTO_JS_LIBRARIES is disabled.
+// [compile+link]
 var STRICT = 0;
 
+// Add "use strict;" to generated JS
+var STRICT_JS = 0;
+
 // If set to 1, we will warn on any undefined symbols that are not resolved by
 // the library_*.js files. Note that it is common in large projects to not
 // implement everything, when you know what is not going to actually be called
@@ -770,14 +870,6 @@ var WARN_ON_UNDEFINED_SYMBOLS = 1;
 // be reported.
 var ERROR_ON_UNDEFINED_SYMBOLS = 1;
 
-// My default any -lfoo directives pointing to nonexistent library files will
-// issue a linker error.  Set to 0 to downgrade this to a warning.
-var ERROR_ON_MISSING_LIBRARIES = 1;
-
-// Specifies a list of Emscripten-provided JS libraries to link against.
-// (internal, use -lfoo or -lfoo.js to link to Emscripten system JS libraries)
-var SYSTEM_JS_LIBRARIES = [];
-
 // Use small chunk size for binary synchronous XHR's in Web Workers.  Used for
 // testing.  See test_chunked_synchronous_xhr in runner.py and library.js.
 var SMALL_XHR_CHUNKS = 0;
@@ -879,6 +971,11 @@ var SEPARATE_ASM_MODULE_NAME = '';
 // be enabled for ES6 exports.
 var EXPORT_ES6 = 0;
 
+// Use the ES6 Module relative import feature 'import.meta.url'
+// to auto-detect WASM Module path.
+// It might not be supported on old browsers / toolchains
+var USE_ES6_IMPORT_META = 1;
+
 // If 1, will just time how long main() takes to execute, and not print out
 // anything at all whatsoever. This is useful for benchmarking.
 var BENCHMARK = 0;
@@ -904,11 +1001,6 @@ var SWAPPABLE_ASM_MODULE = 0;
 // see emcc --separate-asm
 var SEPARATE_ASM = 0;
 
-// This disables linking and other causes of adding extra code automatically,
-// and as a result, your output compiled code (in the .asm.js file, if you emit
-// with --separate-asm) will contain only the functions you provide.
-var ONLY_MY_CODE = 0;
-
 // JS library functions on this list are not converted to JS, and calls to them
 // are turned into abort()s. This is potentially useful for reducing code size.
 // If a dead function is actually called, you will get a runtime error.
@@ -982,9 +1074,6 @@ var EMTERPRETIFY_SYNCLIST = [];
 // whether js opts will be run, after the main compiler
 var RUNNING_JS_OPTS = 0;
 
-// whether we are emitting JS glue code
-var EMITTING_JS = 1;
-
 // whether we are in the generate struct_info bootstrap phase
 var BOOTSTRAPPING_STRUCT_INFO = 0;
 
@@ -1000,11 +1089,38 @@ var EMSCRIPTEN_TRACING = 0;
 var USE_GLFW = 2;
 
 // Whether to use compile code to WebAssembly. Set this to 0 to compile to
-// asm.js.  This will fetch the binaryen port and build it. (If, instead, you
-// set BINARYEN_ROOT in your ~/.emscripten file, then we use that instead of the
-// port, which can useful for local dev work on binaryen itself).
+// asm.js in fastcomp, or JS in upstream.
+//
+// Note that in upstream, WASM=0 behaves very similarly to WASM=1, in particular
+// startup can be either async or sync, so flags like WASM_ASYNC_COMPILATION
+// still make sense there, see that option for more details.
 var WASM = 1;
 
+// STANDALONE_WASM indicates that we want to emit a wasm file that can run without
+// JavaScript. The file will use standard APIs such as wasi as much as possible
+// to achieve that.
+//
+// This option does not guarantee that the wasm can be used by itself - if you
+// use APIs with no non-JS alternative, we will still use those (e.g., OpenGL
+// at the time of writing this). This gives you the option to see which APIs
+// are missing, and if you are compiling for a custom wasi embedding, to add
+// those to your embedding.
+//
+// We may still emit JS with this flag, but the JS should only be a convenient
+// way to run the wasm on the Web or in Node.js, and you can run the wasm by
+// itself without that JS (again, unless you use APIs for which there is no
+// non-JS alternative) in a wasm runtime like wasmer or wasmtime.
+//
+// Note that even without this option we try to use wasi etc. syscalls as much
+// as possible. What this option changes is that we do so even when it means
+// a tradeoff with JS size. For example, when this option is set we do not
+// import the Memory - importing it is useful for JS, so that JS can start to
+// use it before the wasm is even loaded, but in wasi and other wasm-only
+// environments the expectation is to create the memory in the wasm itself.
+// Doing so prevents some possible JS optimizations, so we only do it behind
+// this flag.
+var STANDALONE_WASM = 0;
+
 // Whether to use the WebAssembly backend that is in development in LLVM.  You
 // should not set this yourself, instead set EMCC_WASM_BACKEND=1 in the
 // environment.
@@ -1014,17 +1130,22 @@ var WASM_BACKEND = 0;
 // of using LLVM IR.
 // Setting to zero will enable LTO and at link time will also enable bitcode
 // versions of the standard libraries.
+// [compile+link]
 var WASM_OBJECT_FILES = 1;
 
 // An optional comma-separated list of script hooks to run after binaryen,
 // in binaryen's /scripts dir.
 var BINARYEN_SCRIPTS = "";
 
-// Whether to ignore implicit traps when optimizing in binaryen.  Implicit traps
-// are the unlikely traps that happen in a load that is out of bounds, or
-// div/rem of 0, etc. We can reorder them, but we can't ignore that they have
-// side effects, so turning on this flag lets us do a little more to reduce code
-// size.
+// Whether to ignore implicit traps when optimizing in binaryen.  Implicit
+// traps are the traps that happen in a load that is out of bounds, or
+// div/rem of 0, etc. With this option set, the optimizer assumes that loads
+// cannot trap, and therefore that they have no side effects at all. This
+// is *not* safe in general, as you may have a load behind a condition which
+// ensures it it is safe; but if the load is assumed to not have side effects it
+// could be executed unconditionally. For that reason this option is generally
+// not useful on large and complex projects, but in a small and simple enough
+// codebase it may help reduce code size a little bit.
 var BINARYEN_IGNORE_IMPLICIT_TRAPS = 0;
 
 // How we handle wasm operations that may trap, which includes integer
@@ -1068,6 +1189,14 @@ var WASM_MEM_MAX = -1;
 // not block the main thread. This is currently required for all but the
 // smallest modules to run in chrome.
 //
+// Note that this flag is still useful even if WASM=0 when using the upstream
+// backend, as startup behaves the same there as WASM=1 (the implementation is
+// of a fake WebAssembly.* object, so the startup code doesn't know it's JS
+// and not wasm). That makes it easier to swap between JS and wasm builds,
+// however, this is a difference from fastcomp in which WASM=0 always meant
+// sync startup as asm.js (unless a mem init file was used or some other thing
+// that forced async).
+//
 // (This option was formerly called BINARYEN_ASYNC_COMPILATION)
 var WASM_ASYNC_COMPILATION = 1;
 
@@ -1125,6 +1254,9 @@ var USE_LIBPNG = 0;
 // 1 = use Regal from emscripten-ports
 var USE_REGAL = 0;
 
+// 1 = use Boost headers from emscripten-ports
+var USE_BOOST_HEADERS = 0;
+
 // 1 = use bullet from emscripten-ports
 var USE_BULLET = 0;
 
@@ -1159,11 +1291,19 @@ var SDL2_IMAGE_FORMATS = [];
 var IN_TEST_HARNESS = 0;
 
 // If true, enables support for pthreads.
+// [compile+link] - affects user code at compile and system libraries at link
 var USE_PTHREADS = 0;
 
-// Specifies the number of web workers that are preallocated before runtime is
-// initialized. If 0, workers are created on demand.
+// PTHREAD_POOL_SIZE specifies the number of web workers that are created
+// before the main runtime is initialized. If 0, workers are created on
+// demand. If PTHREAD_POOL_DELAY_LOAD = 0, then the workers will be fully
+// loaded (available for use) prior to the main runtime being initialized. If
+// PTHREAD_POOL_DELAY_LOAD = 1, then the workers will only be created and
+// have their runtimes loaded on demand after the main runtime is initialized.
+// Note that this means that the workers cannot be joined from the main thread
+// unless PROXY_TO_PTHREAD is used.
 var PTHREAD_POOL_SIZE = 0;
+var PTHREAD_POOL_DELAY_LOAD = 0;
 
 // If not explicitly specified, this is the stack size to use for newly created
 // pthreads.  According to
@@ -1188,6 +1328,9 @@ var PTHREADS_PROFILING = 0;
 // If true, add in debug traces for diagnosing pthreads related issues.
 var PTHREADS_DEBUG = 0;
 
+// If true, building against Emscripten's asm.js/wasm heap memory profiler.
+var MEMORYPROFILER = 0;
+
 var MAX_GLOBAL_ALIGN = -1; // received from the backend
 var IMPLEMENTED_FUNCTIONS = []; // received from the backend
 
@@ -1286,6 +1429,13 @@ var FETCH_DEBUG = 0;
 // If nonzero, enables emscripten_fetch API.
 var FETCH = 0;
 
+// Whether to use an asm.js fetch worker when using FETCH. Note that this is
+// only relevant for fastcomp, where we support asm.js. As a result, some
+// synchronous fetch operations that depend on the fetch worker may not work
+// with the wasm backend, like waiting or IndexedDB.
+// Currently will always be set to 0 on WASM backend.
+var USE_FETCH_WORKER = 1;
+
 // Internal: name of the file containing the Fetch *.fetch.js, if relevant
 // Do not set yourself.
 var FETCH_WORKER_FILE = '';
@@ -1311,10 +1461,20 @@ var SINGLE_FILE = 0;
 // to execute the file without the accompanying JS file.
 var EMIT_EMSCRIPTEN_METADATA = 0;
 
+// If set to 1, all JS libraries will be automaticially available at link time.
+// This gets set to 0 in STRICT mode (or with MINIMAL_RUNTIME) which mean you
+// need to explictly specify -lfoo.js in at link time in order to access
+// library function in library_foo.js.
+var AUTO_JS_LIBRARIES = 1;
+
 //==============================
 // Internal use only, from here
 //==============================
 
+// Specifies a list of Emscripten-provided JS libraries to link against.
+// (internal, use -lfoo or -lfoo.js to link to Emscripten system JS libraries)
+var SYSTEM_JS_LIBRARIES = [];
+
 // This will contain the emscripten version. You should not modify this. This
 // and the following few settings are useful in combination with
 // RETAIN_COMPILER_SETTINGS
@@ -1323,6 +1483,9 @@ var EMSCRIPTEN_VERSION = '';
 // This will contain the optimization level (-Ox). You should not modify this.
 var OPT_LEVEL = 0;
 
+// Will be set to 0 if -fno-rtti is used on the command line.
+var USE_RTTI = 1;
+
 // This will contain the debug level (-gx). You should not modify this.
 var DEBUG_LEVEL = 0;
 
@@ -1335,7 +1498,8 @@ var EMIT_SYMBOL_MAP = 0;
 // tracks the list of EM_ASM signatures that are proxied between threads.
 var PROXIED_FUNCTION_SIGNATURES = [];
 
-var ORIGINAL_EXPORTED_FUNCTIONS = [];
+// List of function explictly exported by user on the command line.
+var USER_EXPORTED_FUNCTIONS = [];
 
 // name of the file containing wasm text, if relevant
 var WASM_TEXT_FILE = '';
@@ -1388,11 +1552,13 @@ var SUPPORT_ERRNO = 1;
 // support library. This does not need to be set directly, but pass -fno-exceptions
 // to the build disable exceptions support. (This is basically -fno-exceptions, but
 // checked at final link time instead of individual .cpp file compile time)
+// If the program *does* contain throwing code (some source files were not compiled
+// with `-fno-exceptions`), and this flag is set at link time, then you will get
+// errors on undefined symbols, as the exception throwing code is not linked in. If
+// so you should either unset the option (if you do want exceptions) or fix the
+// compilation of the source files so that indeed no exceptions are used).
 var DISABLE_EXCEPTION_THROWING = 0;
 
-// Internal: An array of all symbols exported from asm.js/wasm module.
-var MODULE_EXPORTS = [];
-
 // Internal (testing only): Disables the blitOffscreenFramebuffer VAO path.
 var OFFSCREEN_FRAMEBUFFER_FORBID_VAO_PATH = 0;
 
@@ -1426,9 +1592,6 @@ var USES_DYNAMIC_ALLOC = 1;
 // be beneficial.
 var RUNTIME_FUNCS_TO_IMPORT = ['abort', 'setTempRet0', 'getTempRet0']
 
-// Internal: stores the base name of the output file (-o TARGET_BASENAME.js)
-var TARGET_BASENAME = '';
-
 // If true, compiler supports setjmp() and longjmp(). If false, these APIs are not available.
 // If you are using C++ exceptions, but do not need setjmp()+longjmp() API, then you can set
 // this to 0 to save a little bit of code size and performance when catching exceptions.
@@ -1446,41 +1609,22 @@ var DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR = 0;
 // to explicitly choose to disable HTML minification altogether.
 var MINIFY_HTML = 1;
 
-// Indicates that the syscalls (which we see statically) indicate that they need full
-// filesystem support. Otherwise, when just a small subset are used, we can get away without
-// including the full filesystem - in particular, if open() is never used, then we don't
-// actually need to support operations on streams.
-var SYSCALLS_REQUIRE_FILESYSTEM = 1;
+// Whether we *may* be using wasm2js. This compiles to wasm normally, but lets you
+// run wasm2js *later* on the wasm, and you can pick between running the normal
+// wasm or that wasm2js code. For details of how to do that, see the test_maybe_wasm2js
+// test.
+// This option can be useful for debugging and bisecting.
+var MAYBE_WASM2JS = 0;
 
-// A list of feature flags to pass to each binaryen invocation (like wasm-opt, etc.). This
-// is received from wasm-emscripten-finalize, which reads it from the features section.
-var BINARYEN_FEATURES = [];
+// The size of our shadow memory.
+// By default, we have 32 MiB. This supports 256 MiB of real memory.
+var ASAN_SHADOW_SIZE = 33554432;
 
-// Whether EMCC_AUTODEBUG is on, which automatically instruments code for runtime
-// logging that can help in debugging.
-var AUTODEBUG = 0;
-
-// Whether we should use binaryen's wasm2js to convert our wasm to JS. Set when
-// wasm backend is in use with WASM=0 (to enable non-wasm output, we compile to
-// wasm normally, then compile that to JS).
-var WASM2JS = 0;
-
-// Whether we should link in the runtime for ubsan.
-// 0 means do not link ubsan, 1 means link minimal ubsan runtime.
-// This is not meant to be used with `-s`. Instead, to use ubsan, use clang flag
-// -fsanitize=undefined. To use minimal runtime, also pass `-fsanitize-minimal-runtime`.
-var UBSAN_RUNTIME = 0;
-
-// Whether we should link in LSan's runtime library. This is intended to be used invoked`
-// by -fsanitize=leak instead of used directly.
-var USE_LSAN = 0;
-
-// Whether we should load the WASM source map at runtime.
-// This is enabled automatically when using -g4 with sanitizers.
-var LOAD_SOURCE_MAP = 0;
-
-// Whether embind has been enabled.
-var EMBIND = 0;
+// Whether we should use the offset converter.
+// This is needed for older versions of v8 (<7.7) that does not give the hex module offset
+// into wasm binary in stack traces, as well as for avoiding using source map
+// entries across function boundaries.
+var USE_OFFSET_CONVERTER = 0;
 
 // Legacy settings that have been removed or renamed.
 // For renamed settings the format is:
@@ -1505,4 +1649,7 @@ var LEGACY_SETTINGS = [
   ['BINARYEN_METHOD', ['native-wasm'], 'Starting from Emscripten 1.38.23, Emscripten now always builds either to Wasm (-s WASM=1 - default), or to asm.js (-s WASM=0), other methods are not supported (https://github.com/emscripten-core/emscripten/pull/7836)'],
   ['PRECISE_I64_MATH', [1, 2], 'Starting from Emscripten 1.38.26, PRECISE_I64_MATH is always enabled (https://github.com/emscripten-core/emscripten/pull/7935)'],
   ['MEMFS_APPEND_TO_TYPED_ARRAYS', [1], 'Starting from Emscripten 1.38.26, MEMFS_APPEND_TO_TYPED_ARRAYS=0 is no longer supported. MEMFS no longer supports using JS arrays for file data (https://github.com/emscripten-core/emscripten/pull/7918)'],
+  ['ERROR_ON_MISSING_LIBRARIES', [1], 'missing libraries are always an error now'],
+  ['EMITTING_JS', [1], 'The new STANDALONE_WASM flag replaces this (replace EMITTING_JS=0 with STANDALONE_WASM=1)'],
+  ['SKIP_STACK_IN_SMALL', [0, 1], 'SKIP_STACK_IN_SMALL is no longer needed as the backend can optimize it directly'],
 ];
