@@ -1,5 +1,11 @@
-#ifndef __emscripten_vector_h__
-#define __emscripten_vector_h__
+/*
+ * Copyright 2013 The Emscripten Authors.  All rights reserved.
+ * Emscripten is available under two separate licenses, the MIT license and the
+ * University of Illinois/NCSA Open Source License.  Both these licenses can be
+ * found in the LICENSE file.
+ */
+
+#pragma once
 
 #if (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) || defined(__cplusplus)
 #define EM_INLINE inline
@@ -21,7 +27,7 @@ extern "C" {
 // and all functions in the list order specified in the EcmaScript SIMD.js specification
 // so that cross-referencing changes against the implementation and the specification is much easier.
 
-// Also, if changing this file, remember to do the matching changes in LLVM side at https://github.com/kripken/emscripten-fastcomp/blob/incoming/lib/Target/JSBackend/CallHandlers.h
+// Also, if changing this file, remember to do the matching changes in LLVM side at https://github.com/emscripten-core/emscripten-fastcomp/blob/incoming/lib/Target/JSBackend/CallHandlers.h
 
 typedef double float64x2 __attribute__((__vector_size__(16), __may_alias__));
 typedef float float32x4 __attribute__((__vector_size__(16), __may_alias__));
@@ -470,5 +476,3 @@ int emscripten_bool8x16_allTrue(bool8x16 __a) __attribute__((__nothrow__, __cons
 #ifdef __cplusplus
 }
 #endif
-
-#endif
